/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.tiff.provider;

import java.text.Format;
import it.tidalwave.image.metadata.TIFF;
import it.tidalwave.metadata.text.APEXApertureFormat;
import it.tidalwave.metadata.text.APEXShutterFormat;
import it.tidalwave.metadata.text.ApertureFormat;
import it.tidalwave.metadata.text.CFAPatternFormat;
import it.tidalwave.metadata.text.DeweyFormat;
import it.tidalwave.metadata.text.EVFormat;
import it.tidalwave.metadata.text.EXIFFlashFormat;
import it.tidalwave.metadata.text.FocalFormat;
import it.tidalwave.metadata.text.ISOFormat;
import it.tidalwave.metadata.text.ShutterFormat;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class TIFFFormatProviderTest 
  {
    private TIFFFormatProvider formatProvider;
    private TIFF tiff;
    
    @Before
    public void setupFixture()
      {
        formatProvider = new TIFFFormatProvider();
        tiff = new TIFF();
      }
    
    @After
    public void tearDownFixture()
      {
        formatProvider = null;
        tiff = null;
      }
    
    @Test
    public void testGetItemClass() 
      {
        assertEquals(TIFF.class, formatProvider.getItemClass());
      }

    @Test
    public void testGetItemDisplayName() 
      {
        assertEquals("TIFF", formatProvider.getItemDisplayName());
      }
    
    @Test
    public void testGetDisplayName()
      {
        assertEquals("Aperture", formatProvider.getDisplayName("apertureValue"));
        assertEquals("\u0192 Number", formatProvider.getDisplayName("FNumber"));
        // stop: we can't test hundreds of items
 
      }
    
    @Test
    public void testGetDisplayNameWithUndefinedKey()
      {
        assertEquals("xxxx", formatProvider.getDisplayName("xxxx"));
      }
    
    @Test
    public void testGetFormat()
      {
        assertFormat(formatProvider.getFormat("apertureValue"), APEXApertureFormat.class);
        assertFormat(formatProvider.getFormat("maxApertureValue"), APEXApertureFormat.class);
        assertFormat(formatProvider.getFormat("exposureBiasValue"), EVFormat.class);
        assertFormat(formatProvider.getFormat("exposureTime"), ShutterFormat.class);
        assertFormat(formatProvider.getFormat("FNumber"), ApertureFormat.class);
        assertFormat(formatProvider.getFormat("ISOSpeedRatings"), ISOFormat.class);
        assertFormat(formatProvider.getFormat("focalLength"), FocalFormat.class);
        assertFormat(formatProvider.getFormat("focalLength35mmFilm"), FocalFormat.class);
        assertFormat(formatProvider.getFormat("shutterSpeedValue"), APEXShutterFormat.class);
        assertFormat(formatProvider.getFormat("EXIFCFAPattern"), CFAPatternFormat.class);
        assertFormat(formatProvider.getFormat("EXIFVersion"), DeweyFormat.class);
        assertFormat(formatProvider.getFormat("DNGBackwardVersion"), DeweyFormat.class);
        assertFormat(formatProvider.getFormat("DNGVersion"), DeweyFormat.class);
        assertFormat(formatProvider.getFormat("TIFF_EPStandardID"), DeweyFormat.class);
        assertFormat(formatProvider.getFormat("flashPixVersion"), DeweyFormat.class);
        assertFormat(formatProvider.getFormat("flash"), EXIFFlashFormat.class);
      }
    
    private static void assertFormat (final Format format, final Class<?> clazz)
      {
        assertTrue(format instanceof RationalFormatWrapper);
        final Class<? extends Format> delegateClass = ((RationalFormatWrapper) format).getDelegate().getClass();
        assertTrue("" + delegateClass, clazz.isAssignableFrom(delegateClass));
      }
  }