/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.tiff.provider;

import java.beans.PropertyVetoException;
import java.util.Date;
import java.io.IOException;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import it.tidalwave.image.metadata.TIFF;
import it.tidalwave.metadata.MetadataItemHolder;
import static it.tidalwave.metadata.Metadata.StorageType.*;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class TIFFProviderTest 
  {
    private TIFFProvider tiffProvider;
    
    @Before
    public void setUp()
      throws PropertyVetoException, IOException 
      {
        tiffProvider = new TIFFProvider();
      }

    @After
    public void tearDown() 
      {
        tiffProvider = null;
      }

    @Test
    public void testGetManagedClass() 
      {
        assertEquals(TIFF.class, tiffProvider.getItemClass());
      }  

    @Test
    public void testFindMetadataItemFromGoodFile()
      throws Exception
      {
        final DataObject dataObject = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("goodfile"));       
        final MetadataItemHolder<TIFF> holder = tiffProvider.findOrCreateItems(dataObject, EXTERNAL).iterator().next();
        assertNotNull(holder);
        assertTrue(holder.isAvailable());
      }
    
    @Test
    public void testFindMetadataItemFromBadJPG()
      throws Exception
      {
        final DataObject dataObject = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("badfile"));       
        final MetadataItemHolder<TIFF> holder = tiffProvider.findOrCreateItems(dataObject, EXTERNAL).iterator().next();
        assertNotNull(holder);
        assertFalse(holder.isAvailable());
      }
    
    @Test
    public void testLastestModificationTimeIsUpdated()
      throws Exception
      {
        final DataObject dataObject = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("goodfile"));       
        final MetadataItemHolder<TIFF> holder = tiffProvider.findOrCreateItems(dataObject, EXTERNAL).iterator().next();
        assertNotNull(holder);
        final Date timestamp1 = holder.getLatestModificationTime();
        assertTrue(holder.isAvailable());
        
        Thread.sleep(1000);
        final Date now = new Date();
        holder.get().setArtist("new artist");
        final Date timestamp2 = holder.getLatestModificationTime();
        assertTrue(timestamp2.getTime() - now.getTime() < 50);
        assertNotSame(timestamp1, timestamp2);
      }
  }